import { ICrossFieldConditions, IFormLayout, IFormV2 } from './models';
import { EnumType, ICrossFieldValidations } from './modelsCommon';
import { IFormSettings, IFormParsed } from './modelsParsed';

export const parseFormV2 = <T extends EnumType>(form: IFormV2<T>): IFormParsed<T> => {
  const {
    submitButtonText,
    cancelButtonText,
    validationObject,
    crossFieldValidations,
    crossFieldConditions,
    formLayout,
    children,
  } = form;

  const allFieldNames = Object.keys(validationObject.properties) as T[];

  const formOptions = allFieldNames.reduce((acc, fieldName) => {
    const fieldLayout = formLayout[fieldName] || {
      md: 12,
      sm: 12,
      xs: 12,
    };

    acc[fieldName] = {
      crossFieldValidations: crossFieldValidations?.[fieldName],
      crossFieldConditions: crossFieldConditions?.[fieldName],
      fieldLayout,
    };

    return acc;
  }, {} as IFormSettings<T>);

  return {
    formatVersion: 'parsed',
    jsonSchema: validationObject,
    formSettings: formOptions,
    children,
    submitButtonText,
    cancelButtonText,
  };
};

export const parseToFormV2 = <T extends EnumType>(form: IFormParsed<T>): IFormV2<T> => {
  const { submitButtonText, cancelButtonText, formSettings, jsonSchema, children } = form;

  const allFieldNames = Object.keys(formSettings) as T[];

  const crossFieldValidations: ICrossFieldValidations<T> = {};

  const crossFieldConditions: ICrossFieldConditions<T> = {};

  const formLayout: IFormLayout<T> = {};

  allFieldNames.forEach((key) => {
    crossFieldConditions[key] = formSettings[key]?.crossFieldConditions;
    crossFieldValidations[key] = formSettings[key]?.crossFieldValidations;
    formLayout[key] = formSettings[key]?.fieldLayout || {
      md: 12,
      sm: 12,
      xs: 12,
    };
  });

  return {
    formatVersion: 2,
    crossFieldConditions,
    crossFieldValidations,
    formLayout,
    validationObject: jsonSchema,
    children,
    submitButtonText,
    cancelButtonText,
  };
};
